==> enhancer-data-bridge-1.2.8.tgz: package/controllers/userauth.js <==
// Mock user. 
// Copy from Bodhi/core/user object.

var mocker = require('../mock');

// / test logout
// / replace enhancer with mocker in customer interface and js procedure.
// / regression Test for mock enhancer in customer interface and procedure.
// / consider what is needed to do to keep login env in bodhi is consist with it in dev env.

var UserAuthController = {
    login: function(req, res, next) {
        var uid = req.params.uid;
        var pwd = req.params.pwd;
        var code = req.body.code;
        var context = req.body.context || '{}';
        try {
            context = JSON.parse(context);
        } catch (e) {
            res.send({
                success: false,
                message: '[Dev Error] Invalid Login context.'
            });
            return;
        }

        var loginSettings = req.body.loginSettings;
        try {
            loginSettings = JSON.parse(loginSettings);
        } catch (e) {
            res.send({
                success: false,
                message: '[Dev Error] Invalid login settings in workbench.'
            });
            return;
        }
        
        var dbConfig = req.body.database;
        try {
            dbConfig = JSON.parse(dbConfig);
        } catch (e) {
            res.send({
                success: false,
                message: '[Dev Error] Invalid database configurations in workbench.'
            });
            return
        }

        var serverVars = req.body.serverVars;
        try {
            serverVars = JSON.parse(decodeURIComponent(serverVars));
        } catch(e) {
            serverVars = {};
        }

        // Mock user
        var user = mocker.mockUser(uid, pwd);
        for (var key in context) {
            user.setVariable('LOGIN_CONTEXT_' + key, context[key]);
        }
        req.session.user = user;

        // Mock enhancer
        var enhancer = global.Enhancer = mocker.mockEnhancer(dbConfig, serverVars, req, res);

        // Mock login process
        var before = function(enhancer, user, done) {done()};
        if (loginSettings.beforeAuthen && loginSettings.beforeAuthen.enabled) {
            try {
                var beforeFunc = eval( '(' + loginSettings.beforeAuthen.funcStr + ')' );
                before = function(enhancer, user, done) {
                    try {
                        beforeFunc(enhancer, user, done);
                    } catch(err) {
                        done(err);
                    }
                }
            } catch (e) {
                res.send({
                    success: false,
                    code: 'BEFORE_LOGIN_SYNTAX_ERR',
                    message: e.message
                });
                return;
            }
        }
        var login;
        if (!loginSettings.authen) {
            res.end();
            return
        }
        if (loginSettings.authen.mode === 'userTable') {
            var map = loginSettings.authen.userTableMapping;
            var sql = 'SELECT '
                    + map['USER_ID'] + ' USER_ID, ' 
                    + map['USER_NAME'] + ' USER_NAME, '
                    + map['ROLES'] + ' ROLES, '
                    + 'u.* '
                    + 'FROM ' + map['tableName'] + ' u '
                    + 'WHERE ' + map['USER_ID'] + ' = ? AND ' + map['PASSWORD'] + ' = ?';
            console.log('Authentication mode is userTable.');
            console.log('Authentication sql is "' + sql + '"');
            login = function(Enhancer, user, callback) {
                var uid = user.getId();
                var pwd = user.getPassword();
                var dbService = Enhancer.getDatabaseService();
                dbService.execute(sql, [uid, pwd], function(err, data) {
                    if (err) {
                        return callback(err)
                    }
                    if (!data || !data.rows) {
                        console.error('Unexpected data result when user login.' );
                        console.error('Validate SQL: ', sql);
                        console.error('user: ', uid, ' pwd: ', pwd);
                        console.error('data result: ', data);
                        return callback(null, false);
                    }
                    if (!data.rows.length) {
                        return callback(null, false);
                    }
                    var u = data.rows[0];
                    
                    // Set name and roles for this user.
                    user.setRoles(u.ROLES);
                    user.setName(u.USER_NAME);
                    // Set other fields as user data.
                    user.setVariables(u);

                    // add all
                    callback(null, true);
                });
            };
        } else {
            try {
                var loginFunc = eval( '(' + loginSettings.authen.customFuncStr + ')' );

                login = function(enhancer, user, done) {
                    try {
                        loginFunc(enhancer, user, done);
                    } catch(err) {
                        done(err);
                    }
                };
            } catch (e) {
                res.send({
                    success: false,
                    code: 'LOGIN_SYNTAX_ERR',
                    message: e.message
                });
                return;
            }
        }

        var after = function(enhancer, user, done) {done()};
        if (loginSettings.afterAuthen && loginSettings.afterAuthen.enabled) {
            try {
                var afterFunc = eval( '(' + loginSettings.afterAuthen.funcStr + ')' );
                after = function(enhancer, user, done) {
                    try {
                        afterFunc(enhancer, user, done);
                    } catch(err) {
                        done(err);
                    }
                };
            } catch (e) {
                res.send({
                    success: false,
                    code: 'AFTER_LOGIN_SYNTAX_ERR',
                    message: e.message
                });
                return;
            }
        };
        before(enhancer, user, function(err, isValid) {
            if (err) {
                console.error(err);
                res.send({
                    code: 'BEFORE_LOGIN_ERROR',
                    message: err.message
                });
                return;
            }
            if (isValid === false || typeof isValid === 'string') {
                res.send({
                    success: false,
                    code: isValid === false ? 'INVALID_USER_OR_PWD' : '',
                    message: typeof isValid === 'string' ? isValid : ''
                });
                return;
            }
            login(enhancer, user, function(err, isValid) {
                if (err) {
                    console.error(err);
                    res.send({
                        code: 'LOGIN_ERROR',
                        message: err.message
                    });
                    return;
                }
                if (isValid !== true) {
                    res.send({
                        success: false,
                        code: isValid === false ? 'INVALID_USER_OR_PWD' : '',
                        message: typeof isValid === 'string' ? isValid : ''
                    });
                    return;
                }
                after(enhancer, user, function(err, isValid) {
                    if (err) {
                        console.error(err);
                        res.send({
                            code: 'AFTER_LOGIN_ERROR',
                            message: err.message
                        });
                        return;
                    }
                    if (isValid === false || typeof isValid === 'string') {
                        res.send({
                            success: false,
                            code: isValid === false ? 'INVALID_USER_OR_PWD' : '',
                            message: typeof isValid === 'string' ? isValid : ''
                        });
                        return;
                    }
                    var vars = user.getVariables();
                    vars['USER_NAME'] = user.getName();
                    vars['USER_ID'] = user.getId();
                    for (var i in vars) {
                        vars['1-' + i] = vars[i];
                    }
                    res.send({
                        success: true,
                        variables: vars
                    });
                });
            });
        });
    },
    logout: function(req, res, next) {
        res.jsonp({
            success: true,
            message: 'bye'
        });

        var loginSettings = req.query.loginSettings;
        try {
            loginSettings = JSON.parse(loginSettings);
        } catch (e) {
            console.error('[Dev Error] Invalid login settings in workbench');
            console.error(e);
            return;
        }
        
        if (!loginSettings.logout || !loginSettings.logout.enabled) {
            return;
        }
        var logoutFunc = loginSettings.logout.funcStr;
        try {
            logoutFunc = eval( '(' + logoutFunc + ')');
        } catch (e) {
            console.error('[Dev Error] The logout function has syntax error.');
            console.error(e);
            console.error('Original Function Settings:', logoutFunc);
            return;
        }
        var dbConfig = req.query.database;
        try {
            dbConfig = JSON.parse(dbConfig);
        } catch (e) {
            console.error('[Dev Error] Invalid database configurations in workbench.');
            console.error(e);
            return;
        }
 
        var serverVars = req.query.serverVars;
        try {
            serverVars = JSON.parse(decodeURIComponent(serverVars));
        } catch(e) {
            serverVars = {};
        }

        var enhancer = global.Enhancer = mocker.mockEnhancer(dbConfig, serverVars, req, res);
        var user = mocker.mockUser(serverVars['USER_ID'], '');
        user.setName(serverVars['USER_NAME']);
        user.setRoles(serverVars['ROLES']);

        logoutFunc(enhancer, user);
    }
};

module.exports = UserAuthController;

==> enhancer-data-bridge-1.3.2.tgz: package/controllers/userauth.js <==
// Mock user. 
// Copy from Bodhi/core/user object.

var mocker = require('../mock');

// / test logout
// / replace enhancer with mocker in customer interface and js procedure.
// / regression Test for mock enhancer in customer interface and procedure.
// / consider what is needed to do to keep login env in bodhi is consist with it in dev env.

var UserAuthController = {
    login: function(req, res, next) {
        var uid = req.params.uid;
        var pwd = req.params.pwd;
        var code = req.body.code;
        var context = req.body.context || '{}';
        try {
            context = JSON.parse(context);
        } catch (e) {
            res.send({
                success: false,
                message: '[Dev Error] Invalid Login context.'
            });
            return;
        }

        var loginSettings = req.body.loginSettings;
        try {
            loginSettings = JSON.parse(loginSettings);
        } catch (e) {
            res.send({
                success: false,
                message: '[Dev Error] Invalid login settings in workbench.'
            });
            return;
        }
        
        var dbConfig = req.body.database;
        try {
            dbConfig = JSON.parse(dbConfig);
        } catch (e) {
            res.send({
                success: false,
                message: '[Dev Error] Invalid database configurations in workbench.'
            });
            return
        }

        var serverVars = req.body.serverVars;
        try {
            serverVars = JSON.parse(decodeURIComponent(serverVars));
        } catch(e) {
            serverVars = {};
        }

        // Mock user
        var __enhancer_user = mocker.mockUser(uid, pwd);
        for (var key in context) {
            __enhancer_user.setVariable('LOGIN_CONTEXT_' + key, context[key]);
        }
        req.session.user = __enhancer_user;

        // Mock enhancer
        var __enhancer = global.Enhancer = mocker.mockEnhancer(dbConfig, serverVars, req, res);

        // Mock login process
        var before = function(enhancer, user, done) {done()};
        if (loginSettings.beforeAuthen && loginSettings.beforeAuthen.enabled) {
            try {
                var beforeFunc = eval( '(' + loginSettings.beforeAuthen.funcStr + ')' );
                before = function(enhancer, user, done) {
                    try {
                        beforeFunc(enhancer, user, done);
                    } catch(err) {
                        done(err);
                    }
                }
            } catch (e) {
                res.send({
                    success: false,
                    code: 'BEFORE_LOGIN_SYNTAX_ERR',
                    message: e.message
                });
                return;
            }
        }
        var login;
        if (!loginSettings.authen) {
            res.end();
            return
        }
        if (loginSettings.authen.mode === 'userTable') {
            var map = loginSettings.authen.userTableMapping;
            var sql = 'SELECT '
                    + map['USER_ID'] + ' USER_ID, ' 
                    + map['USER_NAME'] + ' USER_NAME, '
                    + map['ROLES'] + ' ROLES, '
                    + 'u.* '
                    + 'FROM ' + map['tableName'] + ' u '
                    + 'WHERE ' + map['USER_ID'] + ' = ? AND ' + map['PASSWORD'] + ' = ?';
            console.log('Authentication mode is userTable.');
            console.log('Authentication sql is "' + sql + '"');
            login = function(Enhancer, user, callback) {
                var uid = user.getId();
                var pwd = user.getPassword();
                var dbService = Enhancer.getDatabaseService();
                dbService.execute(sql, [uid, pwd], function(err, data) {
                    if (err) {
                        return callback(err)
                    }
                    if (!data || !data.rows) {
                        console.error('Unexpected data result when user login.' );
                        console.error('Validate SQL: ', sql);
                        console.error('user: ', uid, ' pwd: ', pwd);
                        console.error('data result: ', data);
                        return callback(null, false);
                    }
                    if (!data.rows.length) {
                        return callback(null, false);
                    }
                    var u = data.rows[0];
                    
                    // Set name and roles for this user.
                    user.setRoles(u.ROLES);
                    user.setName(u.USER_NAME);
                    // Set other fields as user data.
                    user.setVariables(u);

                    // add all
                    callback(null, true);
                });
            };
        } else {
            try {
                var loginFunc = eval( '(' + loginSettings.authen.customFuncStr + ')' );

                login = function(enhancer, user, done) {
                    try {
                        loginFunc(enhancer, user, done);
                    } catch(err) {
                        done(err);
                    }
                };
            } catch (e) {
                res.send({
                    success: false,
                    code: 'LOGIN_SYNTAX_ERR',
                    message: e.message
                });
                return;
            }
        }

        var after = function(enhancer, user, done) {done()};
        if (loginSettings.afterAuthen && loginSettings.afterAuthen.enabled) {
            try {
                var afterFunc = eval( '(' + loginSettings.afterAuthen.funcStr + ')' );
                after = function(enhancer, user, done) {
                    try {
                        afterFunc(enhancer, user, done);
                    } catch(err) {
                        done(err);
                    }
                };
            } catch (e) {
                res.send({
                    success: false,
                    code: 'AFTER_LOGIN_SYNTAX_ERR',
                    message: e.message
                });
                return;
            }
        };
        before(__enhancer, __enhancer_user, function(err, isValid) {
            if (err) {
                console.error(err);
                res.send({
                    code: 'BEFORE_LOGIN_ERROR',
                    message: err.message
                });
                return;
            }
            if (isValid === false || typeof isValid === 'string') {
                res.send({
                    success: false,
                    code: isValid === false ? 'INVALID_USER_OR_PWD' : '',
                    message: typeof isValid === 'string' ? isValid : ''
                });
                return;
            }
            login(__enhancer, __enhancer_user, function(err, isValid) {
                if (err) {
                    console.error(err);
                    res.send({
                        code: 'LOGIN_ERROR',
                        message: err.message
                    });
                    return;
                }
                if (isValid !== true) {
                    res.send({
                        success: false,
                        code: isValid === false ? 'INVALID_USER_OR_PWD' : '',
                        message: typeof isValid === 'string' ? isValid : ''
                    });
                    return;
                }
                after(__enhancer, __enhancer_user, function(err, isValid) {
                    if (err) {
                        console.error(err);
                        res.send({
                            code: 'AFTER_LOGIN_ERROR',
                            message: err.message
                        });
                        return;
                    }
                    if (isValid === false || typeof isValid === 'string') {
                        res.send({
                            success: false,
                            code: isValid === false ? 'INVALID_USER_OR_PWD' : '',
                            message: typeof isValid === 'string' ? isValid : ''
                        });
                        return;
                    }
                    var vars = __enhancer_user.getVariables();
                    vars['USER_NAME'] = __enhancer_user.getName();
                    vars['USER_ID'] = __enhancer_user.getId();
                    for (var i in vars) {
                        vars['1-' + i] = vars[i];
                    }
                    res.send({
                        success: true,
                        variables: vars
                    });
                });
            });
        });
    },
    logout: function(req, res, next) {
        res.jsonp({
            success: true,
            message: 'bye'
        });

        var loginSettings = req.query.loginSettings;
        try {
            loginSettings = JSON.parse(loginSettings);
        } catch (e) {
            console.error('[Dev Error] Invalid login settings in workbench');
            console.error(e);
            return;
        }
        
        if (!loginSettings.logout || !loginSettings.logout.enabled) {
            return;
        }
        var logoutFunc = loginSettings.logout.funcStr;
        try {
            logoutFunc = eval( '(' + logoutFunc + ')');
        } catch (e) {
            console.error('[Dev Error] The logout function has syntax error.');
            console.error(e);
            console.error('Original Function Settings:', logoutFunc);
            return;
        }
        var dbConfig = req.query.database;
        try {
            dbConfig = JSON.parse(dbConfig);
        } catch (e) {
            console.error('[Dev Error] Invalid database configurations in workbench.');
            console.error(e);
            return;
        }
 
        var serverVars = req.query.serverVars;
        try {
            serverVars = JSON.parse(decodeURIComponent(serverVars));
        } catch(e) {
            serverVars = {};
        }

        var __enhancer = global.Enhancer = mocker.mockEnhancer(dbConfig, serverVars, req, res);
        var __enhancer_user = mocker.mockUser(serverVars['USER_ID'], '');
        __enhancer_user.setName(serverVars['USER_NAME']);
        __enhancer_user.setRoles(serverVars['ROLES']);

        logoutFunc(__enhancer, __enhancer_user);
    }
};

module.exports = UserAuthController;
